# -*- coding: utf-8 -*-
"""
Created on Tue Nov  2 08:59:58 2021

@author: matthijsdejong
"""

import json
import numpy as np
import pandas as pd
from scipy.optimize import curve_fit
import matplotlib.pyplot as plt

def linearcurve(x, a, b):
    return a*x+b

def fit(f, x, y, p0):
    """ Fitting function. It uses scipy's curve_fit with bounds. It outputs the optimal 
    fitting parameters popt, the covariances pcov, the errors on the optimized parameters
    perr, and the r squared r_squared to measure the goodness of the fit. 
    """
    # fit
    popt, pcov = curve_fit(f, x, y, p0=p0)
    # sigma_fit = (y/10)
    # popt, pcov = curve_fit(f, x, y, p0=p0,  sigma=sigma_fit, absolute_sigma=True)

    # print fit parameters
    perr = np.sqrt(np.diag(pcov))
   
    # calculate r^2 of the fit
    residuals = y - f(x, *popt)
    ss_res = np.sum(residuals**2)
    ss_tot = np.sum((y-np.mean(y))**2)
    r_squared = 1 - (ss_res / ss_tot)
    
    return popt, pcov, perr, r_squared

def linear_fit(xdata, ydata):
    a = -1
    b = -60
    
    p0   =   [a,      b]  # initial fitting parameters
    # perform the fit
    popt, pcov, perr, r_squared = fit(linearcurve, xdata, ydata, p0)

    return popt, pcov, perr, r_squared

#%%
R1 = pd.read_csv('184918/Ringdown.csv', header = 0, sep = ',')
measparam = json.load(open("184918/Measurement parameters.json"))
Rd1 = pd.read_csv('174312/Ringdown.csv', header = 0, sep = ',')
measparam2 = json.load(open("174312/Measurement parameters.json"))


#%%
#Fit the two different linear parts of the 2nd mode.
tclow = 2800
tchigh = 6000

popt, pcov, perr, r_squared = linear_fit(Rd1.iloc[tclow:tchigh,5], Rd1.iloc[tclow:tchigh,6])
FitdBm = linearcurve(Rd1.iloc[tclow:tchigh,5], popt[0], popt[1])

decayratelinear = np.log(10)*popt[0]/10
  
linearQ = -(2*measparam2["Peaklist"][2]/(decayratelinear))
Qunca = -(2*measparam2["Peaklist"][2]/(np.log(10)*(popt[0]+perr[0])/10))
Quncb = -(2*measparam2["Peaklist"][2]/(np.log(10)*(popt[0]-perr[0])/10))
Qunc = np.abs(Qunca - Quncb)/2

print("Mode 2 Q1 = {:.0f} +- {:.0f}".format(linearQ,Qunc))

tclow2 = 2250
tchigh2 = 2500

popt2, pcov2, perr2, r_squared2 = linear_fit(Rd1.iloc[tclow2:tchigh2,5], Rd1.iloc[tclow2:tchigh2,6])
FitdBm2 = linearcurve(Rd1.iloc[tclow2:tchigh2,5], popt2[0], popt2[1])

decayratelinear2 = np.log(10)*popt2[0]/10
  
linearQ2 = -(2*measparam2["Peaklist"][2]/(decayratelinear2))
Qunc2a = -(2*measparam2["Peaklist"][2]/(np.log(10)*(popt2[0]+perr2[0])/10))
Qunc2b = -(2*measparam2["Peaklist"][2]/(np.log(10)*(popt2[0]-perr2[0])/10))
Qunc2 = np.abs(Qunc2a - Qunc2b)/2

print("Mode 2 Q2 = {:.0f} +- {:.0f}".format(linearQ2,Qunc2))

#%%
#Fit the linear parts of the first and third mode.
tclow3 = 2250
tchigh3 = 6000

popt3, pcov3, perr3, r_squared3 = linear_fit(R1.iloc[tclow3:tchigh3,1], R1.iloc[tclow3:tchigh3,2])
FitdBm3 = linearcurve(R1.iloc[tclow3:tchigh3,1], popt3[0], popt3[1])

decayratelinear3 = np.log(10)*popt3[0]/10
  
linearQ3 = -(2*measparam["Peaklist"][0]/(decayratelinear3))
Qunca3 = -(2*measparam["Peaklist"][0]/(np.log(10)*(popt3[0]+perr3[0])/10))
Quncb3 = -(2*measparam["Peaklist"][0]/(np.log(10)*(popt3[0]-perr3[0])/10))
Qunc3 = np.abs(Qunca3 - Quncb3)/2

print("Mode 1 Q = {:.0f} +- {:.0f}".format(linearQ3,Qunc3))

tclow4 = 2250
tchigh4 = 6000

popt4, pcov4, perr4, r_squared4 = linear_fit(R1.iloc[tclow4:tchigh4,17], R1.iloc[tclow4:tchigh4,18])
FitdBm4 = linearcurve(R1.iloc[tclow4:tchigh4,17], popt4[0], popt4[1])

decayratelinear4 = np.log(10)*popt4[0]/10
  
linearQ4 = -(2*measparam["Peaklist"][7]/(decayratelinear4))
Qunca4 = -(2*measparam["Peaklist"][7]/(np.log(10)*(popt4[0]+perr4[0])/10))
Quncb4 = -(2*measparam["Peaklist"][7]/(np.log(10)*(popt4[0]-perr4[0])/10))
Qunc4 = np.abs(Qunca4 - Quncb4)/2

print("Mode 3 Q = {:.0f} +- {:.0f}".format(linearQ4,Qunc4))

#%%


fig1 = plt.figure(figsize = (6,2.5))
ax1 = fig1.add_subplot(111)

ax1.set_xlabel('Time (s)')
ax1.set_ylabel('Noise PSD (dB)')

ax1.plot(R1.iloc[:,1]-0.1,R1.iloc[:,2]+68, color = 'C4', alpha = 0.7, label = '114.9 kHz', zorder = 1)
ax1.plot(Rd1.iloc[:,5]-0.1,Rd1.iloc[:,6]+26, alpha = 1, label = '240.3 kHz', zorder = 3)
ax1.plot(R1.iloc[:,17]-0.1,R1.iloc[:,18]+56.5, color = 'C5', alpha = 0.7, label = '360.5 kHz', zorder = 2)

ax1.plot(R1.iloc[tclow3:tchigh3,1]-0.1,FitdBm3+68, color = 'black', zorder = 4, linewidth = 1)
ax1.plot(R1.iloc[tclow4:tchigh4,17]-0.1,FitdBm4+56.5, color = 'black', zorder = 5, linewidth = 1)


#ax1.scatter(Rd1.iloc[:,5],Rd1.iloc[:,6]+27, marker = '.', alpha = 1, s = 10, label = 'Data')
ax1.plot(Rd1.iloc[tclow:tchigh,5]-0.1,FitdBm+26, color = 'C1', zorder = 6) #label = r'Q = {:.1f}$\times 10^5$'.format(linearQ/1e5),
ax1.plot(Rd1.iloc[tclow2:tchigh2,5]-0.1,FitdBm2+26, color = 'C3', zorder = 7)#label = r'Q = {:.1f}$\times 10^4$'.format(linearQ2/1e4),
ax1.legend()
#ax1.set_xlim((9.5,30))
#ax1.set_ylim((-105,-22))

#ax1.plot([10,10],[-60,5], color = 'black', linewidth = 1, zorder = 0)


ax1.set_xlim((9.5,25))
ax1.set_ylim((-71,5))
ax1.legend(loc = 'upper right', ncol = 3, frameon = False, borderaxespad = 0)

fig1.savefig('Comparison Ringdowns.png', bbox_inches = 'tight', dpi = 300)
fig1.savefig('Comparison Ringdowns.svg', bbox_inches = 'tight', format = 'svg')