# -*- coding: utf-8 -*-
"""
Created on Thu Aug 19 15:12:31 2021

@author: matthijsdejong
"""
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from scipy.optimize import curve_fit
from scipy.integrate import odeint
from scipy.signal import stft
from scipy.interpolate import interp1d
import json

def Lorentzian(x, I, x0, gamma, offset):
    #Cauchy-distribution (Lorentzian power spectrum)
    return I / ( 1 + ( ( x-x0 ) / gamma ) ** 2 ) + offset

def linearcurve(x, a, b):
    return a*x+b

def fit(f, x, y, p0):
    """ Fitting function. It uses scipy's curve_fit with bounds. It outputs the optimal 
    fitting parameters popt, the covariances pcov, the errors on the optimized parameters
    perr, and the r squared r_squared to measure the goodness of the fit. 
    """
    # fit
    popt, pcov = curve_fit(f, x, y, p0=p0)
    # sigma_fit = (y/10)
    # popt, pcov = curve_fit(f, x, y, p0=p0,  sigma=sigma_fit, absolute_sigma=True)

    # print fit parameters
    perr = np.sqrt(np.diag(pcov))
   
    # calculate r^2 of the fit
    residuals = y - f(x, *popt)
    ss_res = np.sum(residuals**2)
    ss_tot = np.sum((y-np.mean(y))**2)
    r_squared = 1 - (ss_res / ss_tot)
    
    return popt, pcov, perr, r_squared

def linear_fit(xdata, ydata):
    a = -1
    b = -60
    
    p0   =   [a,      b]  # initial fitting parameters
    # perform the fit
    popt, pcov, perr, r_squared = fit(linearcurve, xdata, ydata, p0)

    return popt, pcov, perr, r_squared

#%%

S1 = pd.read_csv('180327/Spectrum.csv', header = 0, sep = ',')
S2 = pd.read_csv('180327/Zoominspectrum.csv', header = 0, sep = ',')
Rd1 = pd.read_csv('180327/Ringdown.csv', header = 0, sep = ',')
measparam = json.load(open("180327/Measurement parameters.json"))
#%%
spoint = 2751 + np.argmin(Rd1.iloc[2751:2851,2])
cpoint = spoint

minpoints = np.array(spoint)

while cpoint < 8000:
    if cpoint < 3000:
        npoint = cpoint+5 + np.argmin(Rd1.iloc[cpoint+5:cpoint+160,2])
    elif cpoint < 4000:
        npoint = cpoint+5 + np.argmin(Rd1.iloc[cpoint+5:cpoint+110,2])
    else:
        npoint = cpoint+5 + np.argmin(Rd1.iloc[cpoint+5:cpoint+70,2])
    minpoints = np.append(minpoints,npoint)
    cpoint = npoint

#%%

fig2 = plt.figure(figsize = (6,3))
ax2 = fig2.add_subplot(111)

ax2.set_xlabel('Time (s)')
ax2.set_ylabel('Noise PSD (dBm)')

ax2.plot(Rd1.iloc[:,1],Rd1.iloc[:,2], label = 'Data')
ax2.scatter(Rd1.iloc[minpoints,1],Rd1.iloc[minpoints,2], color = 'red', marker = '.', label = 'Minimum')
ax2.legend()
ax2.set_xlim((12.3,31))
ax2.set_ylim((-90,-25))

#fig2.savefig('Ringdown 180327.png', bbox_inches = 'tight', dpi = 300)
#fig2.savefig('Ringdown 180327.svg', bbox_inches = 'tight', format = 'svg')

#%%

tmins = Rd1.iloc[minpoints,1]
fdiff = tmins.diff().pow(-1)

fsim = 5 - 5*np.exp(-0.21*(tmins-12.5))
fsim2 = 5*(1 - np.exp(-(tmins - 12.5)/4.76))
#fdiff2 = 5.00*(1 - np.exp(-(tmins -12.5)/13.15*np.e))


fig3 = plt.figure(figsize = (6,2.5))
ax3 = fig3.add_subplot(111)

ax3.set_xlabel('Time (s)')
ax3.set_ylabel('Frequency difference (Hz)')

ax3.scatter(Rd1.iloc[minpoints[1:-1],1]-12.7,fdiff.iloc[1:-1])
ax3.plot(tmins-12.7,fsim, color = 'C1')
ax3.plot(tmins-12.7,fsim2, color = 'C2')
#ax3.plot(tmins-12.7,fdiff2, color = 'C3')

ax3.set_ylim((0,6.8))
ax3.set_xlim((0,18))


#fig3.savefig('Frequency spacing.png', bbox_inches = 'tight', dpi = 300)
#fig3.savefig('Frequency spacing.svg', bbox_inches = 'tight', format = 'svg')

#%%
tclow = 6000
tchigh = 10000

popt, pcov, perr, r_squared = linear_fit(Rd1.iloc[tclow:tchigh,1], Rd1.iloc[tclow:tchigh,2])
FitdBm = linearcurve(Rd1.iloc[tclow:tchigh,1], popt[0], popt[1])

decayratelinear = np.log(10)*popt[0]/10
  
linearQ = -(2*measparam["Peaklist"][0]/(decayratelinear))
Qunca = -(2*measparam["Peaklist"][0]/(np.log(10)*(popt[0]+perr[0])/10))
Quncb = -(2*measparam["Peaklist"][0]/(np.log(10)*(popt[0]-perr[0])/10))
Qunc = np.abs(Qunca - Quncb)/2

print("Q1 = {:.0f} +- {:.0f}".format(linearQ,Qunc))

tclow2 = 2800
tchigh2 = 4000

popt2, pcov2, perr2, r_squared2 = linear_fit(Rd1.iloc[tclow2:tchigh2,1], Rd1.iloc[tclow2:tchigh2,2])
FitdBm2 = linearcurve(Rd1.iloc[tclow2:tchigh2,1], popt2[0], popt2[1])

decayratelinear2 = np.log(10)*popt2[0]/10
  
linearQ2 = -(2*measparam["Peaklist"][0]/(decayratelinear2))
Qunc2a = -(2*measparam["Peaklist"][0]/(np.log(10)*(popt2[0]+perr2[0])/10))
Qunc2b = -(2*measparam["Peaklist"][0]/(np.log(10)*(popt2[0]-perr2[0])/10))
Qunc2 = np.abs(Qunc2a - Qunc2b)/2

print("Q2 = {:.0f} +- {:.0f}".format(linearQ2,Qunc2))


fig2 = plt.figure(figsize = (6,2))
ax2 = fig2.add_subplot(111)

ax2.set_xlabel('Time (s)')
ax2.set_ylabel('Power (dBm)')

ax2.scatter(Rd1.iloc[:,1],Rd1.iloc[:,2], marker = '.', alpha = 1, s = 10, label = 'Data')
ax2.plot(Rd1.iloc[tclow:tchigh,1],FitdBm, color = 'C1', label = r'Q = {:.1f}$\times 10^5$'.format(linearQ/1e5))
ax2.plot(Rd1.iloc[tclow2:tchigh2,1],FitdBm2, color = 'C3', label = r'Q = {:.1f}$\times 10^5$'.format(linearQ2/1e5))
ax2.legend()
ax2.set_xlim((12.5,45))
ax2.set_ylim((-90,-25))

fig2.savefig('Ringdown fit 180327.png', bbox_inches = 'tight', dpi = 300)
fig2.savefig('Ringdown fit 180327.svg', bbox_inches = 'tight', format = 'svg')

#%%
meff = 10e-12 #Effective mass in kg (estimate)
omega_1 = 2*np.pi*240302.5 #Angular resonance frequency of mode 1 (degenerate)
omega_2 = 2*np.pi*240307.5
gamma_1 = 2*np.pi*0.35*2
gamma_2 = 2*np.pi*0.109*2 #Decay rate of mode


omega = 2*np.pi*np.linspace(240.28e3,240.33e3,10001)
#thermal driving power depends on gamma because thermodynamics
#See e.g. https://qsm.quantumtinkerer.tudelft.nl/3_noise_in_HOs/
FD1 = 9e-15*gamma_1
FD2 = 9e-15*gamma_2

#Power spectral density of thermally-driven resonators.
PSD1 = FD1 / (meff)**2 /((omega**2 - omega_1**2)**2 + (omega*gamma_1)**2)
PSD2 = FD2 / (meff)**2 /((omega**2 - omega_2**2)**2 + (omega*gamma_2)**2)

fig3 = plt.figure()
ax3 = fig3.add_subplot(111)

ax3.set_xlabel('Frequency (kHz)')
ax3.set_ylabel('Power (dBm)')

R1 = 5500
R2 = 6500
ax3.scatter(S2.iloc[R1:R2,1]/1e3,S2.iloc[R1:R2,3], marker = '.', label = 'Driven')
ax3.scatter(S2.iloc[R1:R2,1]/1e3,S2.iloc[R1:R2,2], marker = '.', color = 'green', label = 'Thermal')

DE = 0.02#0.1#0.05
ax3.plot(omega/(2*np.pi*1e3),10*np.log10(PSD1*DE), color = 'black', alpha = 0.5)
ax3.plot(omega/(2*np.pi*1e3),10*np.log10(PSD2), color = 'black', alpha = 0.5)
ax3.plot(omega/(2*np.pi*1e3),10*np.log10(PSD1*DE+PSD2), color = 'black', label = 'Unequal DE')
ax3.plot(omega/(2*np.pi*1e3),10*np.log10(PSD1+PSD2), color = 'black', linestyle = 'dotted', label = 'Equal DE')

offs = 60
ax3.plot(omega/(2*np.pi*1e3),10*np.log10(PSD1*DE+PSD2)-offs, color = 'black')
ax3.plot(omega/(2*np.pi*1e3),10*np.log10(PSD1+PSD2)-offs, color = 'black', linestyle = 'dotted')

ax3.set_xlim(240.285,240.33)
ax3.legend(loc = 'upper right')

fig3.savefig('Comparison spectra zoomin DE.png', dpi = 300, bbox_inches = 'tight')
fig3.savefig('Comparison spectra zoomin DE.svg', format = 'svg', bbox_inches = 'tight')