# -*- coding: utf-8 -*-
"""
Created on Mon Mar 21 16:12:02 2022

@author: matthijsdejong
"""

import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from scipy.optimize import curve_fit
from scipy.integrate import odeint
from scipy.signal import stft
from scipy.interpolate import interp1d
import json

def doubleres(y, t, params):
    x1,v1,x2,v2 = y
    omega_1,gamma_1,omega_2, gamma_2,J = params
    derivs = [v1,-gamma_1*v1-omega_1**2*x1 - J**2*x2,v2,-gamma_2*v2-omega_2**2*x2-J**2*x1]
    return derivs

#S1 = pd.read_csv('174312/Spectrum.csv', header = 0, sep = ',')
#S2 = pd.read_csv('174312/Zoominspectrum.csv', header = 0, sep = ',')
Rd1 = pd.read_csv('180327/Ringdown.csv', header = 0, sep = ',')
measparam = json.load(open("180327/Measurement parameters.json"))
#%%
#omega_1 = 240302.5 #resonance frequency, in Hz
#omega_2 = 240307.5
gamma_1 = 0.35*2 #Decay rate, in angular Hz?
gamma_2 = 0.109*2
J = 20.00#680#4e5 #3e3

#Input the total splitting observed by the beating pattern
splittot = 5.0

#Take the average frequency as input
omega_1 = 240305.0
#Calculate the splitting based on the coupling
nu = np.abs(np.real(splittot**2/2 + 1j*gamma_1/2 - 1j*gamma_2/2 - np.sqrt((gamma_1*1j + omega_1**2)*splittot**2 - J**4))/omega_1)

omega_1 = 240305.0 - nu/2 #240302.5 #resonance frequency of resonator 1, in Hz
omega_2 = 240305.0 + nu/2 #240307.5 

x10 = 21.000 #Initial displacement
v10 = 0 #Initial velocity
x20 = 0.079 #1.36
v20 = 0

params = [omega_1,gamma_1,omega_2,gamma_2,J]
y0 = [x10,v10,x20,v20]

time = 2*np.pi/omega_1
pointsperosc = 20
nperiods = 5*150000
npts = pointsperosc*nperiods
deltat = time/pointsperosc
t = np.linspace(0.,time*nperiods,npts)

psol = odeint(doubleres,y0,t,args = (params,))
#%%
px1 = pd.Series(psol[:,0])

#Use scipy.signal.stft to perform a moving window FFT. 
fsim,tsim,Sxxsim = stft(px1,1/deltat, nperseg = 65536, return_onesided = True)

#Convert amplitude to power
Sxx2sim = np.abs(Sxxsim)
#Extract the main (dominant) frequency
Peakidsim = int(np.argmax(Sxx2sim[0:17000,0]))

RDfullsim = Sxx2sim[Peakidsim,:]


px2 = pd.Series(psol[:,2])

#Use scipy.signal.stft to perform a moving window FFT. 
fsim2,tsim2,Sxxsim2 = stft(px2,1/deltat, nperseg = 65536, return_onesided = True)

#Convert amplitude to power
Sxx2sim2 = np.abs(Sxxsim2)
#Extract the main (dominant) frequency
Peakidsim2 = int(np.argmax(Sxx2sim2[0:17000,0]))

RDfullsim2 = Sxx2sim2[Peakidsim2,:]
#%%
fdiff = 5.00*(1 - np.exp(-(tsim[2:-2]+3.30)/13.15))
#Jplot = J*(1-np.exp(-2.5*t[0:-1:1001]))

fig3 = plt.figure(figsize = (6,3))
ax3 = fig3.add_subplot(111)

ax3.set_xlabel('Time (s)')
ax3.set_ylabel('Power (dB)')
DE = 20 #Detection efficiency of mode 2 wrt mode 1

#ax4 = ax3.twinx()
#ax4.set_ylabel('$J^2$')
#ax4.plot(t[0:-1:1001],Jplot**2, color = 'black', linestyle = 'dotted')

ax3.plot(Rd1.iloc[2822:7172,1]-12.7,Rd1.iloc[2822:7172,2],label = 'Data')

sigtot = 20*np.log10(np.abs(np.exp(1j*2*np.pi*omega_1*tsim[2:-2])*RDfullsim[2:-2] + np.exp(1j*2*np.pi*(omega_1+fdiff)*tsim[2:-2])*DE*RDfullsim2[2:-2]))
ax3.plot(tsim[2:-2],sigtot-52, color = 'black', label = 'Simulation', linewidth = 1)
ax3.plot(tsim[2:-2],20*np.log10(RDfullsim[2:-2])-52, color = 'C3', label = 'Mode 1', alpha = 0.7)
ax3.plot(tsim[2:-2],20*np.log10(DE*RDfullsim2[2:-2])-52, color = 'C1', label = 'Mode 2', alpha = 0.7)


ax3.legend(loc = 'lower left', handlelength = 1.5, labelspacing = 0.3, borderaxespad = 0.2, handletextpad = 0.6)

ax3.set_ylim((-90,-25))
ax3.set_xlim((0,18))

fig3.savefig('Ringing Ringdown 180327 Simulation.png', dpi = 300, bbox_inches = 'tight')
fig3.savefig('Ringing Ringdown 180327 Simulation.svg', format = 'svg', bbox_inches = 'tight')

#%%
fig4 = plt.figure(figsize = (3,1.5))
ax4 = fig4.add_subplot(111)

ax4.plot(Rd1.iloc[2822:7172,1]-12.7,Rd1.iloc[2822:7172,2],label = 'Data')
ax4.plot(tsim[2:-2],sigtot-52, color = 'black', linewidth = 1, label = 'Simulation')

ax4.set_ylim((-90,-50))
ax4.set_xlim((8,14))

ax4.set_xticks([8,10,12,14])
ax4.set_xticklabels(['','','',''])

ax4.set_yticks([-50,-60,-70,-80,-90])
ax4.set_yticklabels(['','','','',''])

fig4.savefig('Ringing Ringdown 180327 Simulation inset.png', dpi = 300, bbox_inches = 'tight')
fig4.savefig('Ringing Ringdown 180327 Simulation inset.svg', format = 'svg', bbox_inches = 'tight')

#%%

spoint = 2751 + np.argmin(Rd1.iloc[2751:2851,2])
cpoint = spoint

minpoints = np.array(spoint)

while cpoint < 8000:
    if cpoint < 3000:
        npoint = cpoint+5 + np.argmin(Rd1.iloc[cpoint+5:cpoint+160,2])
    elif cpoint < 4000:
        npoint = cpoint+5 + np.argmin(Rd1.iloc[cpoint+5:cpoint+110,2])
    else:
        npoint = cpoint+5 + np.argmin(Rd1.iloc[cpoint+5:cpoint+70,2])
    minpoints = np.append(minpoints,npoint)
    cpoint = npoint
    
tmins = Rd1.iloc[minpoints,1]
fdiffdata = tmins.diff().pow(-1)

#fsim = 5 - 5*np.exp(-0.21*(tmins-12.5))

fdiffsim = 5.00*(1 - np.exp(-(tsim[2:-2]+0.2)/13.15*np.e))
alpha1 = -4e-9
alpha2 = -5e-4
meff = 2e-12

#This ringdoown has a thermal noise floor at least below -90 dB, probably around -100 dB. 
#Assuming the thermal motion is about 1-2 pm, -30 dB corresponds to 1-2 nm motional amplitude. 

#max amplitude of RDfullsim (resonator 1) is 10
#max amplitude of RDfullsim (resonator 2) is 0.04
#Likely we hit mode 1's noise floor first (20 times more efficient detection of mode 2)
#Based on the detector efficiency (-6 dB between 2 wrt 1) and the 20 dB gap, I expect mode 2 
#starts with an amplitude of 100-200 pm. 
#Converting simulated amplitude to 100 pm amplitude gives a factor 400000000^2
#Thus the actual alpha should be alpha2 / 400000000^2
#Giving -3e-21 m^-2 s^-2

omega_nl1 = omega_1 + 3*alpha1/(meff*omega_1)*RDfullsim[2:-2]**2
omega_nl2 = omega_2 + 3*alpha2/(meff*omega_2)*RDfullsim2[2:-2]**2
omega_diff = np.abs(omega_nl1 - omega_nl2)

fig5 = plt.figure(figsize = (6,2.5))
ax5 = fig5.add_subplot(111)

ax5.set_xlabel('Time (s)')
ax5.set_ylabel('Frequency difference (Hz)')

ax5.scatter(Rd1.iloc[minpoints[1:-1],1]-12.7,fdiffdata.iloc[1:-1], label = 'Data')
#ax5.plot(tmins-12.7,fsim, color = 'C1')
#ax5.plot(tsim[2:-2],fdiffsim, label = 'Exp. fit', color = 'black', alpha = 0.7)
#ax5.plot(tsim[2:-2],omega_diff, label = 'NL theory')
ax5.plot(tsim[2:-2],omega_nl1-omega_1+5, label = 'Mode 1', color = 'C3', linestyle = 'dashed')
ax5.plot(tsim[2:-2],omega_nl2-omega_2+5, label = 'Mode 2', color = 'C1', linestyle = 'dashed')

ax5.set_xlim(0,18)
ax5.legend()

fig5.savefig('Frequency spacing and model.png', bbox_inches = 'tight', dpi = 300)
fig5.savefig('Frequency spacing and model.svg', bbox_inches = 'tight', format = 'svg')