'''
Created 2019-09-27
@author Matthijs de Jong, modified by Andrea Cupertino on 2020-09-08

This script is intended to plot and save the spectrum of the device under test in the mechanics setup with and without driving

'''

import sys
import os
import shutil
import time
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import json

# %%

raw_data = True #if True, data are saved directly from the sa, if False from the script


base_path = os.path.abspath(os.getcwd())
path = base_path
path = base_path+"\\"+"data\spectrum_log\Spectrum\\20210726_MLPNCWC8#05\\163500_22_outerbeamLEFT_120kHz_RBW19mHz"
# path2 = base_path+"\\"+"data\spectrum_log\Spectrum\\20201015_MX1mm\\134947"
print("Selected path is:", path,". Confirm to continue (Y/N)")
input1 = input()
while input1 == "N" or input1 != "Y":
    if input1 == "N":
        sys.exit("you selected no. Modify the path and run the script again")
    print("Write the right command Y or N")
    input1 = input()
    
diranalysis = path
# diranalysis2 = path2

# import spectrum data and parameters. For raw data (exported from the SA), the settings are inside the csv file. For data acquired from script, the settings are stored in a json file 
if raw_data == True:
    spectrum = pd.read_csv(diranalysis + '\Spectrum.csv',skiprows=31,sep = ';')
    spectrum.columns = ['Freq(Hz)','Amplitude(dBm)noDriving','Amplitude(dBm)noDriving2']
    settings = pd.read_csv(diranalysis + '\Spectrum.csv',nrows =31,header = 0,sep = ';') #red first lines containing the settings
    parameters = settings[settings.columns[0]].to_dict() #convert dataframe into a dict, to be then saved in a json file
elif raw_data == False:
    spectrum = pd.read_csv(diranalysis + '\Spectrum.csv', ',')
    parameters = json.load(open(diranalysis+"\Measurement parameters.json","r"))

#%%

fig1 = plt.figure()
ax1 = fig1.add_subplot(111)          
ax1.set_xlabel("f (kHz)")
ax1.set_ylabel("Power (dBm)")

# ax1.plot(spectrum["Freq(Hz)"]/1e3,spectrum["Amplitude(dBm)noise{}V".format(parameters["SGnoiseamplitude"])], '-',linewidth='0.5')
# ax1.plot(spectrum2["Freq(Hz)"]/1e3,spectrum2["Amplitude(dBm)noise{}V".format(parameters["SGnoiseamplitude"])], '-',linewidth='0.5')

ax1.plot(spectrum["Freq(Hz)"]/1e3,spectrum["Amplitude(dBm)noDriving"], '-',linewidth='0.5')
# ax1.plot(spectrum["Freq(Hz)"]/1e3,spectrum["Amplitude(dBm)noise{}V".format(parameters["SGnoiseamplitude"])], '-',linewidth='0.5')
# ax1.plot(spectrum["Freq(Hz)"]/1e3,spectrum["Amplitude(dBm)sin{}V".format(parameters["SGsinamplitude"])],'-',linewidth='0.5')                             
# ax1.plot(spectrum["Freq(Hz)"]/1e3,spectrum["Amplitude(dBm)sinOFF{}V".format(parameters["SGsinamplitude"])],'-',linewidth='0.5')                             
# ax1.legend(['no driving','noise {} V'.format(parameters["SGnoiseamplitude"]),'sin ON {} V'.format(parameters["SGsinamplitude"]),'sin OFF {} V'.format(parameters["SGsinamplitude"])])
# plt.title("Spectrum averaged {} counts at {} kHz".format(parameters["SAaveragecount"],parameters["SAcentralfrequency"]/1e3))
# ax1.set_ylim(ymin=-135)
# ax1.axvspan(1510, 2730, color='C1', alpha=0.2)

# normalized plot
spectrum ["NormalizedPower(a.u.)"]= (spectrum["Amplitude(dBm)noDriving"]-spectrum["Amplitude(dBm)noDriving"].max())

fig2= plt.figure(figsize=(3.2,2.4))
ax1 = fig2.add_subplot(111)          
ax1.set_xlabel("f (kHz)")
ax1.set_ylabel("Power [a.u.]")
ax1.plot(spectrum["Freq(Hz)"]/1e3,spectrum ["NormalizedPower(a.u.)"], 'k-',linewidth='0.5')
plt.yticks((0, -20, -40, -60, -80), ('10\u2070', '10\u207B\u00B2', '10\u207B\u2074', '10\u207B\u2076','10\u207B\u2078' ))
ax1.set_xlim(xmin=70)

#save the figures and the csv and close them
fig1.savefig(diranalysis + "\Spectrum.png", bbox_inches = 'tight', dpi = 300)
fig2.savefig(diranalysis + "\Spectrum_normalized.png", bbox_inches = 'tight', dpi = 300)
plt.close(fig1)
# spectrum.to_csv(diranalysis+'\Spectrum.csv')

fname = 'Spectrum_plot.py'
if not os.access(diranalysis, os.F_OK):
    os.mkdir(diranalysis, 0o700)
shutil.copy(fname, diranalysis) 

