# -*- coding: utf-8 -*-
"""
Created on Thu Oct 28 11:53:53 2021

@author: matthijsdejong
"""

import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from scipy.optimize import curve_fit
from scipy.signal import savgol_filter

def linearcurve(x, a, b):
    return a*x+b

def Autocorrel(times, omega_0, gamma, scale):
    omega_1 = np.sqrt(omega_0**2 - gamma**2)
    Cxx = np.exp(-gamma*times) * (np.cos(omega_1*times) + gamma/omega_1 * np.sin(omega_1*times))
    return scale*Cxx

def fit(f, x, y, p0):
    """ Fitting function. It uses scipy's curve_fit with bounds. It outputs the optimal 
    fitting parameters popt, the covariances pcov, the errors on the optimized parameters
    perr, and the r squared r_squared to measure the goodness of the fit. 
    """
    # fit
    popt, pcov = curve_fit(f, x, y, p0=p0)
    # sigma_fit = (y/10)
    # popt, pcov = curve_fit(f, x, y, p0=p0,  sigma=sigma_fit, absolute_sigma=True)

    # print fit parameters
    perr = np.sqrt(np.diag(pcov))
   
    # calculate r^2 of the fit
    residuals = y - f(x, *popt)
    ss_res = np.sum(residuals**2)
    ss_tot = np.sum((y-np.mean(y))**2)
    r_squared = 1 - (ss_res / ss_tot)
    
    return popt, pcov, perr, r_squared

Ring1 = pd.read_csv('Ringdown.csv', header = 30, sep = ';')
Ring2 = pd.read_csv('Ringdown2.csv', header = 30, sep = ';')

#%%
kB = 1.38e-23 #Boltzmann constant, m^2 kg s^-2 K^-1
T = 300 #Temperature in K
m = 29e-9 #Mass in kg
omega_0 = 0.17 #120e3#0.6 #mechanical frequency
gamma = 0.04 #Decay rate

times = np.linspace(0,71,5001)

omega_1 = np.sqrt(omega_0**2 - gamma**2)
Cxx = np.exp(-gamma*times) * (np.cos(omega_1*times) + gamma/omega_1 * np.sin(omega_1*times))

Cxx2 = np.exp(-gamma*times) * (1 + gamma*times)

#Get separate part at the start of the measurement, to confirm the autocorrelation.
P1 = 200
P2 = P1+200
    
datcut = Ring1.iloc[P1:P2,1]
timcut = Ring1.iloc[P1:P2,0]

p0 = (-0.1,np.mean(datcut))
popt,pcov,perr,r_squared = fit(linearcurve,timcut,datcut,p0)
callin = linearcurve(timcut,*popt)
fltdata = datcut - callin

Verbose = True
if Verbose == True:
    fig2 = plt.figure()
    ax2 = fig2.add_subplot(111)
    
    ax2.plot(timcut,fltdata)
#Take autocorrelation
Correls2 = np.correlate(fltdata,fltdata, mode = 'full')
lendata = int(len(Correls2)/2)

#Cut off only the positive time coefficients
cut_Correls2 = Correls2[lendata-1:-1]

#Filter and normalize to 1
filt_Correls2 = savgol_filter(cut_Correls2,5,1)
norm_Correls2 = filt_Correls2/np.max(filt_Correls2)

#Get separate part at the start of the measurement, to confirm the autocorrelation.
P3 = 800
P4 = P3+200
    
datcut2 = Ring2.iloc[P3:P4,1]
timcut2 = Ring2.iloc[P3:P4,0]

p02 = (-0.1,np.mean(datcut2))
popt2,pcov2,perr2,r_squared2 = fit(linearcurve,timcut2,datcut2,p02)
callin2 = linearcurve(timcut2,*popt2)
fltdata2 = datcut2 - callin2

Verbose = True
if Verbose == True:
    fig2 = plt.figure()
    ax2 = fig2.add_subplot(111)
    
    ax2.plot(timcut2,fltdata2)
#Take autocorrelation
Correls3 = np.correlate(fltdata2,fltdata2, mode = 'full')
lendata2 = int(len(Correls3)/2)

#Cut off only the positive time coefficients
cut_Correls3 = Correls3[lendata2-1:-1]

#Filter and normalize to 1
filt_Correls3 = savgol_filter(cut_Correls3,5,1)
norm_Correls3 = filt_Correls3/np.max(filt_Correls3)

#%%
p0 = [0.2,0.04,0.4]
popt,pcov,perr,r_squared = fit(Autocorrel, timcut-np.min(timcut),norm_Correls2,p0)

autofit = Autocorrel(timcut - np.min(timcut),*popt)

p02 = [0.2,0.04,0.4]
popt2,pcov2,perr2,r_squared2 = fit(Autocorrel, timcut2-np.min(timcut2),norm_Correls3,p02)

autofit2 = Autocorrel(timcut2 - np.min(timcut2),*popt2)

fig1 = plt.figure(figsize = (4,2))
ax1 = fig1.add_subplot(111)

ax1.set_xlabel('Time (s)')
ax1.set_ylabel('Autocorrelation')

#ax1.plot(times,Cxx)
#ax1.plot(times,Cxx2)
ax1.plot([0,61],[0,0], color = 'black', linestyle = 'dotted', alpha = 0.5)
ax1.scatter(timcut-np.min(timcut),norm_Correls2, s = 5, alpha = 0.3)
ax1.plot(timcut - np.min(timcut),autofit,color = 'C0')
ax1.scatter(timcut2-np.min(timcut2),norm_Correls3, s = 5, color = 'green', alpha = 0.3)
ax1.plot(timcut2 - np.min(timcut2),autofit2,color = 'green')

ax1.set_xlim((0,60))

fig1.savefig('Autocorrelations.png', bbox_inches = 'tight', dpi = 300)
fig1.savefig('Autocorrelations.svg', bbox_inches = 'tight', format = 'svg')

cohtime1 = 1/popt[1]
cohtime2 = 1/popt2[1]

cohunc1 = -1/(popt[1]+perr[1]) + 1/(popt[1]-perr[1]) 
cohunc2 = -1/(popt2[1]+perr2[1]) + 1/(popt2[1]-perr2[1])
#ax1.set_title('$\omega = $ {:.3f} Hz, $\gamma = $ {:.3f} Hz'.format(popt[0],popt[1]))

#%%

omega = 0.3
gamma = 0.02
scale = 1

times = np.linspace(0,50,1001)

Ct = Autocorrel(times,omega,gamma,scale)

fig3 = plt.figure()
ax3 = fig3.add_subplot(111)

ax3.set_xlabel('Time (s)')
ax3.set_ylabel('Autocorrelation')

ax3.plot(times,Ct)

