# -*- coding: utf-8 -*-
"""
Created on Tue Mar 22 09:22:53 2022

@author: matthijsdejong
"""

import numpy as np
import pandas as pd
import json
import matplotlib.pyplot as plt

S1 = pd.read_csv('174312/Spectrum.csv', header = 0, sep = ',')
S2 = pd.read_csv('174312/Zoominspectrum.csv', header = 0, sep = ',')
Rd1 = pd.read_csv('174312/Ringdown.csv', header = 0, sep = ',')
measparam = json.load(open("174312/Measurement parameters.json"))

meff = 10e-12 #Effective mass in kg (estimate)
omega_1 = 2*np.pi*240331.9 #Angular resonance frequency of mode 1 (degenerate)
omega_2 = 2*np.pi*240341.5
gamma_1 = 2*np.pi*3
gamma_2 = 2*np.pi*0.35 #Decay rate of mode, FWHM


omega = 2*np.pi*np.linspace(240.3e3,240.38e3,10001)
#thermal driving power depends on gamma because thermodynamics
#See e.g. https://qsm.quantumtinkerer.tudelft.nl/3_noise_in_HOs/
FD1 = 2e-14*gamma_1
FD2 = 2e-14*gamma_2

#Power spectral density of thermally-driven resonators.
PSD1 = FD1 / (meff)**2 /((omega**2 - omega_1**2)**2 + (omega*gamma_1)**2)
PSD2 = FD2 / (meff)**2 /((omega**2 - omega_2**2)**2 + (omega*gamma_2)**2)

#%%
fig1 = plt.figure()
ax1 = fig1.add_subplot(111)

ax1.set_xlabel('Frequency (kHz)')
ax1.set_ylabel('Power (dB)')

R1 = 3001
R2 = 4401
ax1.scatter(S2.iloc[R1:R2,7]/1e3,S2.iloc[R1:R2,9]+7, marker = '.', alpha = 0.5, label = 'Data')

DE = 0.1#0.1#0.05
ax1.plot(omega/(2*np.pi*1e3),10*np.log10(PSD1*DE), label = 'Resonator 1')
ax1.plot(omega/(2*np.pi*1e3),10*np.log10(PSD2), label = 'Resonator 2')
ax1.plot(omega/(2*np.pi*1e3),10*np.log10(PSD1*DE+PSD2), label = 'Combined')

ax1.legend()
ax1.set_ylim((-85,-30))

#%%
fig2 = plt.figure()
ax2 = fig2.add_subplot(111)

ax2.set_xlabel('Frequency (kHz)')
ax2.set_ylabel('Power (dB)')

R1 = 3001
R2 = 4401
ax2.scatter(S2.iloc[R1:R2,7]/1e3,S2.iloc[R1:R2,8]+7, marker = '.', alpha = 0.5, label = 'Data')

DE = 0.1#0.1#0.05
offs = 63
ax2.plot(omega/(2*np.pi*1e3),10*np.log10(PSD1*DE)-offs, label = 'Resonator 1')
ax2.plot(omega/(2*np.pi*1e3),10*np.log10(PSD2)-offs, label = 'Resonator 2')
ax2.plot(omega/(2*np.pi*1e3),10*np.log10(PSD1*DE+PSD2)-offs, label = 'Combined')

ax2.legend()
ax2.set_ylim((-85-offs,-30-offs))

#%%
R1 = 3001
R2 = 4401

fig3 = plt.figure(figsize = (6,4))
ax3 = fig3.add_subplot(111)

ax3.set_xlabel('Frequency (kHz)')
ax3.set_ylabel('Power (dB)')

ax3.scatter(S2.iloc[R1:R2,7]/1e3,S2.iloc[R1:R2,9], color = 'C0', marker = '.', alpha = 0.5, label = 'Driven', rasterized = True)
ax3.scatter(S2.iloc[R1:R2,7]/1e3,S2.iloc[R1:R2,8], color = 'green', marker = '.', alpha = 0.5, label = 'Thermal', rasterized = True)

DE = 0.05#0.1#0.05
ax3.plot(omega/(2*np.pi*1e3),10*np.log10(PSD1*DE), color = 'black', alpha = 0.5)
ax3.plot(omega/(2*np.pi*1e3),10*np.log10(PSD2), color = 'black', alpha = 0.5)
ax3.plot(omega/(2*np.pi*1e3),10*np.log10(PSD1*DE+PSD2), color = 'black', label = 'Unequal DE')

ax3.plot(omega/(2*np.pi*1e3),10*np.log10(PSD1+PSD2), color = 'black', linestyle = 'dotted', label = 'Equal DE')

offs = 62#63
ax3.plot(omega/(2*np.pi*1e3),10*np.log10(PSD1*DE+PSD2)-offs, color = 'black')
ax3.plot(omega/(2*np.pi*1e3),10*np.log10(PSD1+PSD2)-offs, color = 'black', linestyle = 'dotted')

ax3.legend(loc = 'upper right', ncol = 1)
ax3.set_xlim((240.31,240.37))
ax3.set_ylim((-150,-35))

ax3.set_xticks([240.31,240.32,240.33,240.34,240.35,240.36,240.37])
ax3.set_xticklabels(['-0.03','-0.02','-0.01','240.34','+0.01','+0.02','+0.03'])

fig3.savefig('Spectrum 174312 Near degenerate.png', dpi = 300, bbox_inches = 'tight')
fig3.savefig('Spectrum 174312 Near degenerate.svg', format = 'svg', bbox_inches = 'tight', dpi = 300)