# -*- coding: utf-8 -*-
"""
Created on Mon Mar 21 16:12:02 2022

@author: matthijsdejong
"""

import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from scipy.optimize import curve_fit
from scipy.integrate import odeint
from scipy.signal import stft
from scipy.interpolate import interp1d
import json

def doubleres(y, t, params):
    x1,v1,x2,v2 = y
    omega_1,gamma_1,omega_2, gamma_2,J = params
    derivs = [v1,-gamma_1*v1-omega_1**2*x1 - J**2*x2,v2,-gamma_2*v2-omega_2**2*x2-J**2*x1]
    return derivs

#S1 = pd.read_csv('174312/Spectrum.csv', header = 0, sep = ',')
#S2 = pd.read_csv('174312/Zoominspectrum.csv', header = 0, sep = ',')
Rd1 = pd.read_csv('174312/Ringdown.csv', header = 0, sep = ',')
measparam = json.load(open("174312/Measurement parameters.json"))
uncparam = json.load(open("Optimization results 174312_v7.json"))
#%%
#omega_1 = 240341.5 #resonance frequency, in Hz
#omega_2 = 240331.9
gamma_1 = 0.34*2 #Decay rate, in angular Hz?
gamma_2 = 3*2
J = 320.42#680#4e5 #3e3

#Input the total splitting observed by the beating pattern
splittot = 8.6

#Take the average frequency as input
omega_1 = 240336.65

#Calculate the splitting based on the coupling
nu = np.abs(np.real(splittot**2/2 + 1j*gamma_1/2 - 1j*gamma_2/2 - np.sqrt((gamma_1*1j + omega_1**2)*splittot**2 - J**4))/omega_1)

omega_1 = 240336.65+nu/2 #240341.5 #resonance frequency of resonator 1, in Hz
omega_2 = 240336.65-nu/2 #240331.9 

x10 = 0.0890 #Initial displacement
v10 = 0 #Initial velocity
x20 = 0.0712 #1.36
v20 = 0

params = [omega_1,gamma_1,omega_2,gamma_2,J]
y0 = [x10,v10,x20,v20]

time = 2*np.pi/omega_1
pointsperosc = 100
nperiods = 150000
npts = pointsperosc*nperiods
deltat = time/pointsperosc
t = np.linspace(0.,time*nperiods,npts)

psol = odeint(doubleres,y0,t,args = (params,))
#%%
px1 = pd.Series(psol[:,0])

#Use scipy.signal.stft to perform a moving window FFT. 
fsim,tsim,Sxxsim = stft(px1,1/deltat, nperseg = 65536, return_onesided = True)

#Convert amplitude to power
Sxx2sim = np.abs(Sxxsim)
#Extract the main (dominant) frequency
Peakidsim = int(np.argmax(Sxx2sim[0:17000,0]))

RDfullsim = Sxx2sim[Peakidsim,:]


px2 = pd.Series(psol[:,2])

#Use scipy.signal.stft to perform a moving window FFT. 
fsim2,tsim2,Sxxsim2 = stft(px2,1/deltat, nperseg = 65536, return_onesided = True)

#Convert amplitude to power
Sxx2sim2 = np.abs(Sxxsim2)
#Extract the main (dominant) frequency
Peakidsim2 = int(np.argmax(Sxx2sim2[0:17000,0]))

RDfullsim2 = Sxx2sim2[Peakidsim2,:]
#%%
fdiff = 8.6*(1 - np.exp(-0.61*(tsim[2:-2]+0.64)))
#Jplot = J*(1-np.exp(-2.5*t[0:-1:1001]))

fig3 = plt.figure(figsize = (6,2.5))
ax3 = fig3.add_subplot(111)

ax3.set_xlabel('Time (s)')
ax3.set_ylabel('Power (dB)')
DE = 20 #Detection efficiency of mode 2 wrt mode 1

#ax4 = ax3.twinx()
#ax4.set_ylabel('$J^2$')
#ax4.plot(t[0:-1:1001],Jplot**2, color = 'black', linestyle = 'dotted')

ax3.plot(Rd1.iloc[2270:3134,1] ,Rd1.iloc[2270:3134,6]+26,label = 'Data', color = 'C0', marker = '.')
sigtot = 20*np.log10(np.abs(np.exp(1j*2*np.pi*omega_1*tsim[2:-2])*RDfullsim[2:-2] + np.exp(1j*2*np.pi*(omega_1+fdiff)*tsim[2:-2])*DE*RDfullsim2[2:-2]))
ax3.plot(tsim[2:-2]+10.2,sigtot+4, label = 'Fit', color = 'black', linewidth = 1)

ax3.plot(tsim[2:-2]+10.2,20*np.log10(DE*RDfullsim2[2:-2])-10, label = 'Mode 1', color = 'C3', alpha = 0.7)
ax3.plot(tsim[2:-2]+10.2,20*np.log10(RDfullsim[2:-2])-10, label = 'Mode 2', color = 'C1', alpha = 0.7)
ax3.legend(loc = 'lower center', ncol = 4, frameon = False, borderaxespad = 0)
ax3.set_ylim((-65,5))

ax3.set_xlim((10.2,3.9+10.2))

fig3.savefig('Simulation 174312 optimized.png', dpi = 300, bbox_inches = 'tight')
fig3.savefig('Simulation 174312 optimized.svg', format = 'svg', bbox_inches = 'tight')

#%%
popt = uncparam['popt']
perr = uncparam['perr']

print('J = {:.1f}+-{:.2f}'.format(float(popt[0]),float(perr[0])))
print('x10 = {:.4f}+-{:.4f}'.format(float(popt[1]),float(perr[1])))
print('x20 = {:.4f}+-{:.4f}'.format(float(popt[2]),float(perr[2])))

#%%

#Now also try for other couplings

J = 0
params = [omega_1,gamma_1,omega_2,gamma_2,J]
psol = odeint(doubleres,y0,t,args = (params,))
px1 = pd.Series(psol[:,0])

#Use scipy.signal.stft to perform a moving window FFT. 
fsim,tsim,Sxxsim = stft(px1,1/deltat, nperseg = 65536, return_onesided = True)

#Convert amplitude to power
Sxx2sim = np.abs(Sxxsim)
#Extract the main (dominant) frequency
Peakidsim = int(np.argmax(Sxx2sim[0:17000,0]))

RDfullsima = Sxx2sim[Peakidsim,:]


px2 = pd.Series(psol[:,2])

#Use scipy.signal.stft to perform a moving window FFT. 
fsim2,tsim2,Sxxsim2 = stft(px2,1/deltat, nperseg = 65536, return_onesided = True)

#Convert amplitude to power
Sxx2sim2 = np.abs(Sxxsim2)
#Extract the main (dominant) frequency
Peakidsim2 = int(np.argmax(Sxx2sim2[0:17000,0]))

RDfullsim2a = Sxx2sim2[Peakidsim2,:]
fdiff = 8.6*(1 - np.exp(-0.61*(tsim[2:-2]+0.64)))
#%%
fig4 = plt.figure(figsize = (3,2))
ax4 = fig4.add_subplot(111)

ax4.set_xlabel('Time (s)')
#ax4.set_ylabel('Power (dB)')
DE = 20 #Detection efficiency of mode 2 wrt mode 1

#ax4.plot(Rd1.iloc[2270:3134,1] ,Rd1.iloc[2270:3134,6]+22,label = 'Data', color = 'C0', marker = '.')
#ax4.plot(tsim[2:-2]+10.2,20*np.log10(DE*RDfullsim2a[2:-2]), label = 'Mode 1', color = 'C3')
#ax4.plot(tsim[2:-2]+10.2,20*np.log10(RDfullsima[2:-2]), label = 'Mode 2', color = 'C1')

sigtot = 20*np.log10(np.abs(np.exp(1j*2*np.pi*omega_1*tsim[2:-2])*RDfullsima[2:-2] + np.exp(1j*2*np.pi*(omega_1+fdiff)*tsim[2:-2])*DE*RDfullsim2a[2:-2]))
ax4.plot(tsim[2:-2]+10.2,sigtot-10, label = 'Combined', color = 'black', linewidth = 1)

sigtot2 = 20*np.log10(np.abs(RDfullsima[2:-2] + DE*RDfullsim2a[2:-2]))
ax4.plot(tsim[2:-2]+10.2,sigtot2, label = 'Combined', color = 'black', linewidth = 1, linestyle = 'dashed')
#ax4.legend()

ax4.set_xlim((10.2,3.9+10.2))
ax4.set_ylim((-70,0))

fig4.savefig('Ringdown J = 0.png', dpi = 300, bbox_inches = 'tight')
fig4.savefig('Ringdown J = 0.svg', format = 'svg', bbox_inches = 'tight')

#%%
fig7 = plt.figure(figsize = (6,3))
ax7 = fig7.add_subplot(111)

ax7.set_xlabel('Time (s)')
ax7.set_ylabel('Power (dB)')

DE = 20 #Detection efficiency of mode 2 wrt mode 1
sigtot = 20*np.log10(np.abs(RDfullsima[2:-2] + DE*RDfullsim2a[2:-2]))
ax7.plot(tsim[2:-2]+10.2,sigtot, label = 'Combined', color = 'black', linewidth = 1)



#%%
J = 2e3
params = [omega_1,gamma_1,omega_2,gamma_2,J]
psol = odeint(doubleres,y0,t,args = (params,))
px1 = pd.Series(psol[:,0])

#Use scipy.signal.stft to perform a moving window FFT. 
fsim,tsim,Sxxsim = stft(px1,1/deltat, nperseg = 65536, return_onesided = True)

#Convert amplitude to power
Sxx2sim = np.abs(Sxxsim)
#Extract the main (dominant) frequency
Peakidsim = int(np.argmax(Sxx2sim[0:17000,0]))

RDfullsimb = Sxx2sim[Peakidsim,:]


px2 = pd.Series(psol[:,2])

#Use scipy.signal.stft to perform a moving window FFT. 
fsim2,tsim2,Sxxsim2 = stft(px2,1/deltat, nperseg = 65536, return_onesided = True)

#Convert amplitude to power
Sxx2sim2 = np.abs(Sxxsim2)
#Extract the main (dominant) frequency
Peakidsim2 = int(np.argmax(Sxx2sim2[0:17000,0]))

RDfullsim2b = Sxx2sim2[Peakidsim2,:]
fdiff = 8.6*(1 - np.exp(-0.61*(tsim[2:-2]+0.64)))

#%%
fig5 = plt.figure(figsize = (3,2))
ax5 = fig5.add_subplot(111)

ax5.set_xlabel('Time (s)')
#ax5.set_ylabel('Power (dB)')
DE = 20 #Detection efficiency of mode 2 wrt mode 1

#ax4.plot(Rd1.iloc[2270:3134,1] ,Rd1.iloc[2270:3134,6]+22,label = 'Data', color = 'C0', marker = '.')
#ax5.plot(tsim[2:-2]+10.2,20*np.log10(DE*RDfullsim2b[2:-2]), label = 'Mode 1', color = 'C3')
#ax5.plot(tsim[2:-2]+10.2,20*np.log10(RDfullsimb[2:-2]), label = 'Mode 2', color = 'C1')

sigtot = 20*np.log10(np.abs(np.exp(1j*2*np.pi*omega_1*tsim[2:-2])*RDfullsimb[2:-2] + np.exp(1j*2*np.pi*(omega_1+fdiff)*tsim[2:-2])*DE*RDfullsim2b[2:-2]))
ax5.plot(tsim[2:-2]+10.2,sigtot-10, label = 'Combined', color = 'black', linewidth = 1)

sigtot2 = 20*np.log10(np.abs(RDfullsimb[2:-2] + DE*RDfullsim2b[2:-2]))
ax5.plot(tsim[2:-2]+10.2,sigtot2, label = 'Combined', color = 'black', linewidth = 1, linestyle = 'dashed')
#ax5.legend()

ax5.set_xlim((10.2,3.9+10.2))
ax5.set_ylim((-70,0))

fig5.savefig('Ringdown J = 2e3.png', dpi = 300, bbox_inches = 'tight')
fig5.savefig('Ringdown J = 2e3.svg', format = 'svg', bbox_inches = 'tight')

#%%
fig8 = plt.figure(figsize = (6,3))
ax8 = fig8.add_subplot(111)

ax8.set_xlabel('Time (s)')
ax8.set_ylabel('Power (dB)')

DE = 20 #Detection efficiency of mode 2 wrt mode 1
sigtot = 20*np.log10(np.abs(RDfullsimb[2:-2] + DE*RDfullsim2b[2:-2]))
ax8.plot(tsim[2:-2]+10.2,sigtot, label = 'Combined', color = 'black', linewidth = 1)

#%%
J = 1e3
params = [omega_1,gamma_1,omega_2,gamma_2,J]
psol = odeint(doubleres,y0,t,args = (params,))
px1 = pd.Series(psol[:,0])

#Use scipy.signal.stft to perform a moving window FFT. 
fsim,tsim,Sxxsim = stft(px1,1/deltat, nperseg = 65536, return_onesided = True)

#Convert amplitude to power
Sxx2sim = np.abs(Sxxsim)
#Extract the main (dominant) frequency
Peakidsim = int(np.argmax(Sxx2sim[0:17000,0]))

RDfullsimc = Sxx2sim[Peakidsim,:]


px2 = pd.Series(psol[:,2])

#Use scipy.signal.stft to perform a moving window FFT. 
fsim2,tsim2,Sxxsim2 = stft(px2,1/deltat, nperseg = 65536, return_onesided = True)

#Convert amplitude to power
Sxx2sim2 = np.abs(Sxxsim2)
#Extract the main (dominant) frequency
Peakidsim2 = int(np.argmax(Sxx2sim2[0:17000,0]))

RDfullsim2c = Sxx2sim2[Peakidsim2,:]
fdiff = 8.6*(1 - np.exp(-0.61*(tsim[2:-2]+0.64)))
#%%
fig6 = plt.figure(figsize = (3,2))
ax6 = fig6.add_subplot(111)

ax6.set_xlabel('Time (s)')
#ax6.set_ylabel('Power (dB)')
DE = 20 #Detection efficiency of mode 2 wrt mode 1

#ax4.plot(Rd1.iloc[2270:3134,1] ,Rd1.iloc[2270:3134,6]+22,label = 'Data', color = 'C0', marker = '.')
#ax6.plot(tsim[2:-2]+10.2,20*np.log10(DE*RDfullsim2c[2:-2]), label = 'Mode 1', color = 'C3')
#ax6.plot(tsim[2:-2]+10.2,20*np.log10(RDfullsimc[2:-2]), label = 'Mode 2', color = 'C1')

sigtot = 20*np.log10(np.abs(np.exp(1j*2*np.pi*omega_1*tsim[2:-2])*RDfullsimc[2:-2] + np.exp(1j*2*np.pi*(omega_1+fdiff)*tsim[2:-2])*DE*RDfullsim2c[2:-2]))
ax6.plot(tsim[2:-2]+10.2,sigtot-10, label = 'Combined', color = 'black', linewidth = 1)

sigtot2 = 20*np.log10(np.abs(RDfullsimc[2:-2] + DE*RDfullsim2c[2:-2]))
ax6.plot(tsim[2:-2]+10.2,sigtot2, label = 'Combined', color = 'black', linewidth = 1, linestyle = 'dashed')
#ax6.legend()

ax6.set_xlim((10.2,3.9+10.2))
ax6.set_ylim((-70,0))

fig6.savefig('Ringdown J = 1e3.png', dpi = 300, bbox_inches = 'tight')
fig6.savefig('Ringdown J = 1e3.svg', format = 'svg', bbox_inches = 'tight')

#%%
fig9 = plt.figure(figsize = (6,3))
ax9 = fig9.add_subplot(111)

ax9.set_xlabel('Time (s)')
ax9.set_ylabel('Power (dB)')

DE = 20 #Detection efficiency of mode 2 wrt mode 1
sigtot = 20*np.log10(np.abs(RDfullsimc[2:-2] + DE*RDfullsim2c[2:-2]))
ax9.plot(tsim[2:-2]+10.2,sigtot, label = 'Combined', color = 'black', linewidth = 1)


#%%
fig10 = plt.figure(figsize = (3,2))
ax10 = fig10.add_subplot(111)

ax10.set_xlabel('Time (s)')
#ax6.set_ylabel('Power (dB)')
DE = 20 #Detection efficiency of mode 2 wrt mode 1

#ax4.plot(Rd1.iloc[2270:3134,1] ,Rd1.iloc[2270:3134,6]+22,label = 'Data', color = 'C0', marker = '.')
#ax6.plot(tsim[2:-2]+10.2,20*np.log10(DE*RDfullsim2c[2:-2]), label = 'Mode 1', color = 'C3')
#ax6.plot(tsim[2:-2]+10.2,20*np.log10(RDfullsimc[2:-2]), label = 'Mode 2', color = 'C1')

sigtot = 20*np.log10(np.abs(np.exp(1j*2*np.pi*omega_1*tsim[2:-2])*RDfullsim[2:-2] + np.exp(1j*2*np.pi*(omega_1+fdiff)*tsim[2:-2])*DE*RDfullsim2[2:-2]))
ax10.plot(tsim[2:-2]+10.2,sigtot-10, label = 'Combined', color = 'black', linewidth = 1)

sigtot2 = 20*np.log10(np.abs(RDfullsim[2:-2] + DE*RDfullsim2[2:-2]))
ax10.plot(tsim[2:-2]+10.2,sigtot2, label = 'Combined', color = 'black', linewidth = 1, linestyle = 'dashed')
#ax6.legend()

ax10.set_xlim((10.2,3.9+10.2))
ax10.set_ylim((-70,0))

fig10.savefig('Ringdown J = 320.png', dpi = 300, bbox_inches = 'tight')
fig10.savefig('Ringdown J = 320.svg', format = 'svg', bbox_inches = 'tight')
