# -*- coding: utf-8 -*-
"""
Created on Mon Aug 30 15:53:24 2021

@author: matthijsdejong
"""

import numpy as np
import pandas as pd
import matplotlib.pyplot as plt

#I'm simulating the equations of the paper 'Characterization of MEMS resonator Nonlinearities using the ringdown response'
#From Polunin, Yang, Dykman, Kenny & Shaw
#In Journal of Microelectromechanical systems, 25, 2 (2016)
#10.1109/JMEMS.2016.2529296

a0 = 1 #Initial amplitude value
gamma_1 = 0.02 #coefficient of linear friction
gamma_2 = 1 #Coefficient of non-linear friction
t = np.linspace(0,100,10001)

R1 = pd.read_csv('174312/Ringdown.csv', header = 0, sep = ',')

#%% #Sweep of nonlinear coefficient.
fig1 = plt.figure()
ax1 = fig1.add_subplot(111)

ax1.set_xlabel('Time (?)')
ax1.set_ylabel('Amplitude (dB)')

gamma_2s = [0,0.01,0.1,0.5,1,2,5]
for gamma_2 in gamma_2s:
    goft = 1 + 0.25*gamma_2/gamma_1*a0**2*(1-np.exp(-2*gamma_1*t))
    aoft = a0*np.exp(-gamma_1*t)/np.sqrt(goft) #Envelope of vibrational amplitude of resonator

    ax1.plot(t,10*np.log10(aoft), label = '{}'.format(gamma_2))
    
ax1.legend()
ax1.set_title('Amplitude decay for different values of nonlinear friction coefficient')

#%% #Comparison/fit to data.

a0 = 0.000005 #Initial amplitude value
gamma_1 = 0.35 #coefficient of linear friction
gamma_2 = 13000000000000 #Coefficient of non-linear friction
t = np.linspace(0,22,10001)

goft = 1 + 0.25*gamma_2/gamma_1*a0**2*(1-np.exp(-2*gamma_1*t))
aoft = a0*np.exp(-gamma_1*t)/np.sqrt(goft) #Envelope of vibrational amplitude of resonator

fig3 = plt.figure(figsize = (6,2))
ax3 = fig3.add_subplot(111)

ax3.set_xlabel('Time (s)')
ax3.set_ylabel('Mech PSD (dBm)')

ax3.plot(R1.iloc[2200:7000,5],R1.iloc[2200:7000,6], label = 'Data')
ax3.plot(t+10.1,20*np.log10(aoft)+80, label = 'Theory, nonlinear damping')

ax3.legend()
ax3.set_xlim((9.9,20))
ax3.set_ylim((-85,-22))

fig3.savefig('Nonlinear damping fit.png', dpi = 300, bbox_inches = 'tight')
fig3.savefig('Nonlinear damping fit.svg', format = 'svg', bbox_inches = 'tight')


#%% #Calculation of phase (Eq5) 

a0 = 1 #Initial amplitude value
gamma_1 = 0.02 #coefficient of linear friction
gamma_2 = 1 #Coefficient of non-linear friction
omega_0 = 1 #Natural frequency of resonance
gamma = 1 #Duffing nonlinear coefficient
beta = 1 #Quintic nonlinear coefficient
t = np.linspace(0,100,10001)

goft = 1 + 0.25*gamma_2/gamma_1*a0**2*(1-np.exp(-2*gamma_1*t))
phit = 3*gamma/(4*omega_0*gamma_2)*(1- 10*beta*gamma_1/(3*gamma_2))*np.log(goft) + 5*beta*a0**2/(8*omega_0*gamma_2)*(goft-np.exp(-2*gamma_1*t))/goft


fig2 = plt.figure()
ax2 = fig2.add_subplot(111)

ax2.set_xlabel('Time (s)')
ax2.set_ylabel('Phase (?)')

ax2.plot(t,phit)

